// -*- C++ -*-
//
// SSSSVertex.cc is a part of ThePEG - Toolkit for HEP Event Generation
// Copyright (C) 2003-2007 Peter Richardson, Leif Lonnblad
//
// ThePEG is licenced under version 2 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the SSSSVertex class.
//

#include "SSSSVertex.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"

namespace ThePEG {
namespace Helicity {
    
AbstractNoPIOClassDescription<SSSSVertex> SSSSVertex::initSSSSVertex;
// Definition of the static class description member.

void SSSSVertex::Init() {
  
  static ClassDocumentation<SSSSVertex> documentation
    ("The SSSSVertex class is the implementation"
     "of the helicity amplitude for the four scalar vertex"
     "all vertices of trhis type should inherit from it");
}

// evaluate the vertex
Complex SSSSVertex::evaluate(Energy2 q2, const ScalarWaveFunction & sca1,
			     const ScalarWaveFunction & sca2, 
			     const ScalarWaveFunction & sca3, 
			     const ScalarWaveFunction & sca4)
{
  tcPDPtr Psca1 = sca1.getParticle();
  tcPDPtr Psca2 = sca2.getParticle();
  tcPDPtr Psca3 = sca3.getParticle();
  tcPDPtr Psca4 = sca4.getParticle();
  // calculate the coupling
  setCoupling(q2,Psca1,Psca2,Psca3,Psca4);
  Complex norm=getNorm();
  Complex ii(0.,1.);
  // return the answer
  return ii*norm*sca1.wave()*sca2.wave()*sca3.wave()*sca4.wave();
}

// off-shell scalar
ScalarWaveFunction SSSSVertex::evaluate(Energy2 q2, int iopt, tcPDPtr out,
					const ScalarWaveFunction & sca1,
					const ScalarWaveFunction & sca2,
					const ScalarWaveFunction & sca3)
{
  tcPDPtr Psca1 = sca1.getParticle();
  tcPDPtr Psca2 = sca2.getParticle();
  tcPDPtr Psca3 = sca3.getParticle();
  // outgoing momentum 
  Lorentz5Momentum pout = Lorentz5Momentum(sca1.px()+sca2.px()+sca3.px(),
					   sca1.py()+sca2.py()+sca3.py(),
					   sca1.pz()+sca2.pz()+sca3.pz(),
					   sca1.e() +sca2.e() +sca3.e()); 
  // calculate the coupling
  setCoupling(q2,Psca1,Psca2,Psca3,out);
  // wavefunction
  Energy2 p2=pout.m2();
  Complex fact=-getNorm()*sca1.wave()*sca2.wave()*sca3.wave()*
    propagator(iopt,p2,out);
  return ScalarWaveFunction(pout,out,fact);
}

}
}

